@@ -1,17 +1,37 @@
 /*
 ** pnscan.c - Parallell Network Scanner
 **
-** Copyright (c) 2002 Peter Eriksson <pen@lysator.liu.se>
-**
-** This program is free software; you can redistribute it and/or
-** modify it as you wish - as long as you don't claim that you wrote
-** it.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** Copyright (c) 2002-2020, Peter Eriksson <pen@lysator.liu.se>
+** All rights reserved.
+** 
+** Redistribution and use in source and binary forms, with or without
+** modification, are permitted provided that the following conditions are met:
+** 
+** 1. Redistributions of source code must retain the above copyright notice, this
+**    list of conditions and the following disclaimer.
+** 
+** 2. Redistributions in binary form must reproduce the above copyright notice,
+**    this list of conditions and the following disclaimer in the documentation
+**    and/or other materials provided with the distribution.
+** 
+** 3. Neither the name of the copyright holder nor the names of its
+**    contributors may be used to endorse or promote products derived from
+**    this software without specific prior written permission.
+** 
+** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+** SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+** CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+** OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -41,7 +61,7 @@
 #define RESPONSE_MAX_SIZE    1024
 
 
-extern char version[];
+char version[] = PACKAGE_VERSION;
 
 unsigned char *wstr = NULL;
 int   wlen = 0;
@@ -995,35 +1015,39 @@ get_port_range(char *str,
 	       int *first_port,
 	       int *last_port)
 {
-    char first[256], last[256];
+    char first[256];
+    char last[256];
 
 
-    switch (sscanf(str, "%255[^: ] : %255s", first, last))
+    memset(first, 0, sizeof(first));
+    memset(last, 0, sizeof(last));
+  
+    switch (sscanf(str, "%255[^: ] : %255s", first, last)) 
     {
       case 1:
 	if (strcmp(first, "all") == 0)
 	{
-	    *first_port = 1;
-	    *last_port  = 65535;
-	    return 2;
+	  *first_port = 1;
+	  *last_port  = 65535;
+	  return 2;
 	}
-	
+    
 	if (get_service(first, first_port) != 1)
-	    return -1;
-
+	  return -1;
+	
 	*last_port = *first_port;
 	return 1;
 
-      case 2:
-	if (get_service(first, first_port) != 1)
-	    return -1;
-
-	if (get_service(last, last_port) != 1)
-	    return -1;
-
-	return 2;
+    case 2:
+      if (get_service(first, first_port) != 1)
+	return -1;
+      
+      if (get_service(last, last_port) != 1)
+	return -1;
+      
+      return 2;
     }
-    
+
     return -1;
 }
 
@@ -1072,6 +1096,7 @@ main(int argc,
 	    switch (argv[i][j])
 	    {
 	      case '-':
+              case '\0':
 		++i;
 		goto EndOptions;
 		
@@ -1110,8 +1135,13 @@ main(int argc,
 	      case 'w':
 		if (argv[i][2])
 		    wstr = (unsigned char *) strdup(argv[i]+2);
-		else
+		else if (i+1 < argc)
 		    wstr = (unsigned char *) strdup(argv[++i]);
+                else {
+                  fprintf(stderr, "%s: Missing required argument for '-w'\n",
+                          argv[0]);
+                  exit(1);
+                }
 		
 		wlen = deslash(wstr);
 		j = -2;
@@ -1120,8 +1150,14 @@ main(int argc,
 	      case 'W':
 		if (argv[i][2])
 		    wstr = (unsigned char *) strdup(argv[i]+2);
-		else
+		else if (i+1 < argc)
 		    wstr = (unsigned char *) strdup(argv[++i]);
+                else {
+                  fprintf(stderr, "%s: Missing required argument for '-W'\n",
+                          argv[0]);
+                  exit(1);
+                }
+
 		wlen = dehex(wstr);
 		j = -2;
 		break;
@@ -1129,8 +1165,14 @@ main(int argc,
 	      case 'R':
 		if (argv[i][2])
 		    rstr = (unsigned char *) strdup(argv[i]+2);
-		else
+		else if (i+1 < argc)
 		    rstr = (unsigned char *) strdup(argv[++i]);
+                else {
+                  fprintf(stderr, "%s: Missing required argument for '-R\n",
+                          argv[0]);
+                  exit(1);
+                }
+
 		rlen = dehex(rstr);
 		j = -2;
 		break;
@@ -1138,8 +1180,14 @@ main(int argc,
 	      case 'r':
 		if (argv[i][2])
 		    rstr = (unsigned char *) strdup(argv[i]+2);
-		else
+		else if (i+1 < argc)
 		    rstr = (unsigned char *) strdup(argv[++i]);
+                else {
+                  fprintf(stderr, "%s: Missing required argument for '-r'\n",
+                          argv[0]);
+                  exit(1);
+                }
+
 		rlen = deslash(rstr);
 		j = -2;
 		break;
